#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on June 1 2018

@author: kushal

Chatzigeorgiou Group
Sars International Centre for Marine Molecular Biology

GNU GENERAL PUBLIC LICENSE Version 3, 29 June 2007
"""

from PyQt5 import QtWidgets
from ...pyqtgraphCore import LineSegmentROI, ROI
import numpy as np
from .resize import ResizeDialogBox
from ..core.common import ViewerUtils
from math import sqrt
from .image_projections import display_projection


class ImageMenu:
    def __init__(self, viewer_interface):
        assert isinstance(viewer_interface, ViewerUtils)
        self.vi = viewer_interface

        self.crop_roi = None

        self.projection_windows = []

        self.measure_lines = []
        self._drawing_line = False

        self._prev_mouseclick_pos = None

    def reset_scale(self):
        self.vi.update_workEnv()

    def resize(self):
        if not hasattr(self, 'resize_dialog'):
            self.resize_dialog = ResizeDialogBox(self.vi)
        self.resize_dialog.show()

    def crop(self):
        if self.crop_roi is None:
            self.vi.viewer.status_bar_label.showMessage('Create your crop region and then click on "Crop" again in the '
                                                        '"Image" menu to crop to the selected region')
            self.crop_roi = ROI(pos=[0, 0], size=(0.5 * np.mean([self.vi.viewer.image.shape[1],
                                                                 self.vi.viewer.image.shape[2]])), removable=True)
            self.crop_roi.sigRemoveRequested.connect(self.remove_crop_roi)
            self.crop_roi.addScaleHandle([1, 1], [0, 0])
            self.crop_roi.addScaleHandle([0, 0], [0, 0])
            self.vi.viewer.view.addItem(self.crop_roi)
        else:
            self.vi.viewer.status_bar_label.showMessage('Cropping to your selection, please wait...')
            point_l = self.vi.viewer.getImageItem().mapFromScene(self.crop_roi.getSceneHandlePositions()[1][1])
            point_r = self.vi.viewer.getImageItem().mapFromScene(self.crop_roi.getSceneHandlePositions()[0][1])

            pl = (int(point_l.x()), int(point_l.y()))
            pr = (int(point_r.x()), int(point_r.y()))
            self.remove_crop_roi()

            seq = self.vi.viewer.image[:, pl[1]:pr[1], pl[0]:pr[0]]
            self.vi.viewer.workEnv.imgdata.seq = seq.T
            self.vi.update_workEnv()
            self.vi.viewer.status_bar_label.showMessage('Cropping completed!')

    def remove_crop_roi(self):
        self.vi.viewer.view.removeItem(self.crop_roi)
        self.crop_roi = None
        self.vi.viewer.status_bar_label.clearMessage()

    def action_measure_line_invoked(self):
        if self._drawing_line:
            QtWidgets.QMessageBox.warning(
                self.vi.viewer.parent(),
                'Finish drawing previous line',
                'Click on the image to finish drawing the previous line before drawing a new one',
                QtWidgets.QMessageBox.Ok
            )
            return

        self._drawing_line = True

        self.vi.viewer.status_bar_label.showMessage(
            'Click on a point in the image to draw the first point of a line'
        )

        self.vi.viewer.scene.sigMouseClicked.connect(self.draw_measure_line)

    def draw_measure_line(self, ev):
        self._prev_mouseclick_pos = self.vi.viewer.view.mapSceneToView(ev.pos())

        self.vi.viewer.status_bar_label.showMessage(
            'Click on a second point in the image to finish drawing the line'
        )

        self.vi.viewer.scene.sigMouseClicked.disconnect(self.draw_measure_line)
        self.vi.viewer.scene.sigMouseClicked.connect(self._finish_drawing_measure_line)

    def _finish_drawing_measure_line(self, ev):
        measure_line = LineSegmentROI(
            positions=(
                self._prev_mouseclick_pos,
                self.vi.viewer.view.mapSceneToView(ev.pos())
            ),
            removable=True
        )
        self.vi.viewer.view.addItem(measure_line)

        # changing or hovering on the line will display the distances
        measure_line.sigRegionChanged.connect(self.show_measure_line_distance)
        measure_line.sigHoverEvent.connect(self.show_measure_line_distance)

        # deleting the line
        measure_line.sigRemoveRequested.connect(self.measure_lines.remove)
        measure_line.sigRemoveRequested.connect(self.vi.viewer.scene.removeItem)

        # add to list
        self.measure_lines.append(measure_line)

        self.vi.viewer.scene.sigMouseClicked.disconnect(self._finish_drawing_measure_line)

        self.show_measure_line_distance(measure_line)

        self._drawing_line = False

    def show_measure_line_distance(self, line: LineSegmentROI):
        dx = abs(line.listPoints()[0][0] - line.listPoints()[1][0])
        dy = abs(line.listPoints()[0][1] - line.listPoints()[1][1])
        dist = sqrt((dx ** 2 + dy ** 2))

        self.vi.viewer.status_bar_label.showMessage(
            f"Line distance: dx: {dx:.2f} | dy: {dy:.2f} | distance: {dist:.2f}"
        )

    def mean_projection(self):
        self.vi.viewer.status_bar_label.showMessage('Creating Mean Projection of image sequence, please wait...')
        w = display_projection('mean', self.vi.viewer.workEnv.imgdata.seq,
                               self.vi.viewer.ui.label_curr_img_seq_name.text())
        self.projection_windows.append(w)
        self.vi.viewer.status_bar_label.showMessage('Projection displayed.')

    def max_projection(self):
        self.vi.viewer.status_bar_label.showMessage('Creating Max Projection of image sequence, please wait...')
        w = display_projection('max', self.vi.viewer.workEnv.imgdata.seq,
                               self.vi.viewer.ui.label_curr_img_seq_name.text())
        self.projection_windows.append(w)
        self.vi.viewer.status_bar_label.showMessage('Projection displayed.')

    def std_projection(self):
        self.vi.viewer.status_bar_label.showMessage(
            'Creating Standard Deviation Projection of image sequence, please wait...')
        w = display_projection('std', self.vi.viewer.workEnv.imgdata.seq,
                               self.vi.viewer.ui.label_curr_img_seq_name.text())
        self.projection_windows.append(w)
        self.vi.viewer.status_bar_label.showMessage('Projection displayed.')

    def close_projection_windows(self):
        for w in self.projection_windows:
            w.close()



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file './control_widget.ui'
#
# Created by: PyQt5 UI code generator 5.9.2
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets

class Ui_ControlsDock(object):
    def setupUi(self, ControlsDock):
        ControlsDock.setObjectName("ControlsDock")
        ControlsDock.resize(296, 672)
        self.dockWidgetContents = QtWidgets.QWidget()
        self.dockWidgetContents.setObjectName("dockWidgetContents")
        self.verticalLayout_3 = QtWidgets.QVBoxLayout(self.dockWidgetContents)
        self.verticalLayout_3.setObjectName("verticalLayout_3")
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.label = QtWidgets.QLabel(self.dockWidgetContents)
        self.label.setObjectName("label")
        self.horizontalLayout.addWidget(self.label)
        self.comboBox_data_column = ComboBox(self.dockWidgetContents)
        self.comboBox_data_column.setObjectName("comboBox_data_column")
        self.horizontalLayout.addWidget(self.comboBox_data_column)
        self.verticalLayout_3.addLayout(self.horizontalLayout)
        self.checkBoxIncludeUnlabelled = QtWidgets.QCheckBox(self.dockWidgetContents)
        self.checkBoxIncludeUnlabelled.setObjectName("checkBoxIncludeUnlabelled")
        self.verticalLayout_3.addWidget(self.checkBoxIncludeUnlabelled)
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.label_2 = QtWidgets.QLabel(self.dockWidgetContents)
        self.label_2.setObjectName("label_2")
        self.horizontalLayout_2.addWidget(self.label_2)
        self.comboBox_method = ComboBox(self.dockWidgetContents)
        self.comboBox_method.setObjectName("comboBox_method")
        self.horizontalLayout_2.addWidget(self.comboBox_method)
        self.verticalLayout_3.addLayout(self.horizontalLayout_2)
        self.horizontalLayout_3 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_3.setObjectName("horizontalLayout_3")
        self.label_3 = QtWidgets.QLabel(self.dockWidgetContents)
        self.label_3.setObjectName("label_3")
        self.horizontalLayout_3.addWidget(self.label_3)
        self.spinBox_start_offset = QtWidgets.QSpinBox(self.dockWidgetContents)
        self.spinBox_start_offset.setMinimum(-1000)
        self.spinBox_start_offset.setMaximum(1000)
        self.spinBox_start_offset.setObjectName("spinBox_start_offset")
        self.horizontalLayout_3.addWidget(self.spinBox_start_offset)
        self.verticalLayout_3.addLayout(self.horizontalLayout_3)
        self.horizontalLayout_4 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_4.setObjectName("horizontalLayout_4")
        self.label_4 = QtWidgets.QLabel(self.dockWidgetContents)
        self.label_4.setObjectName("label_4")
        self.horizontalLayout_4.addWidget(self.label_4)
        self.spinBox_end_offset = QtWidgets.QSpinBox(self.dockWidgetContents)
        self.spinBox_end_offset.setMinimum(-1000)
        self.spinBox_end_offset.setMaximum(1000)
        self.spinBox_end_offset.setObjectName("spinBox_end_offset")
        self.horizontalLayout_4.addWidget(self.spinBox_end_offset)
        self.verticalLayout_3.addLayout(self.horizontalLayout_4)
        self.horizontalLayout_5 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_5.setObjectName("horizontalLayout_5")
        self.label_5 = QtWidgets.QLabel(self.dockWidgetContents)
        self.label_5.setObjectName("label_5")
        self.horizontalLayout_5.addWidget(self.label_5)
        self.comboBox_DPT_column = ComboBox(self.dockWidgetContents)
        self.comboBox_DPT_column.setObjectName("comboBox_DPT_column")
        self.horizontalLayout_5.addWidget(self.comboBox_DPT_column)
        self.verticalLayout_3.addLayout(self.horizontalLayout_5)
        self.pushButton_set = QtWidgets.QPushButton(self.dockWidgetContents)
        self.pushButton_set.setObjectName("pushButton_set")
        self.verticalLayout_3.addWidget(self.pushButton_set)
        self.pushButton_save = QtWidgets.QPushButton(self.dockWidgetContents)
        self.pushButton_save.setObjectName("pushButton_save")
        self.verticalLayout_3.addWidget(self.pushButton_save)
        self.line = QtWidgets.QFrame(self.dockWidgetContents)
        self.line.setFrameShape(QtWidgets.QFrame.HLine)
        self.line.setFrameShadow(QtWidgets.QFrame.Sunken)
        self.line.setObjectName("line")
        self.verticalLayout_3.addWidget(self.line)
        self.horizontalLayout_6 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_6.setObjectName("horizontalLayout_6")
        self.verticalLayout_2 = QtWidgets.QVBoxLayout()
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.label_6 = QtWidgets.QLabel(self.dockWidgetContents)
        self.label_6.setObjectName("label_6")
        self.verticalLayout_2.addWidget(self.label_6)
        self.listWidget_samples = ListWidget(self.dockWidgetContents)
        self.listWidget_samples.setObjectName("listWidget_samples")
        self.verticalLayout_2.addWidget(self.listWidget_samples)
        self.horizontalLayout_6.addLayout(self.verticalLayout_2)
        self.verticalLayout = QtWidgets.QVBoxLayout()
        self.verticalLayout.setObjectName("verticalLayout")
        self.label_7 = QtWidgets.QLabel(self.dockWidgetContents)
        self.label_7.setObjectName("label_7")
        self.verticalLayout.addWidget(self.label_7)
        self.listWidget_rois = ListWidget(self.dockWidgetContents)
        self.listWidget_rois.setMaximumSize(QtCore.QSize(100, 16777215))
        self.listWidget_rois.setObjectName("listWidget_rois")
        self.verticalLayout.addWidget(self.listWidget_rois)
        self.horizontalLayout_6.addLayout(self.verticalLayout)
        self.verticalLayout_3.addLayout(self.horizontalLayout_6)
        ControlsDock.setWidget(self.dockWidgetContents)

        self.retranslateUi(ControlsDock)
        QtCore.QMetaObject.connectSlotsByName(ControlsDock)
        ControlsDock.setTabOrder(self.comboBox_data_column, self.checkBoxIncludeUnlabelled)
        ControlsDock.setTabOrder(self.checkBoxIncludeUnlabelled, self.comboBox_method)
        ControlsDock.setTabOrder(self.comboBox_method, self.spinBox_start_offset)
        ControlsDock.setTabOrder(self.spinBox_start_offset, self.spinBox_end_offset)
        ControlsDock.setTabOrder(self.spinBox_end_offset, self.comboBox_DPT_column)
        ControlsDock.setTabOrder(self.comboBox_DPT_column, self.pushButton_set)
        ControlsDock.setTabOrder(self.pushButton_set, self.pushButton_save)
        ControlsDock.setTabOrder(self.pushButton_save, self.listWidget_samples)
        ControlsDock.setTabOrder(self.listWidget_samples, self.listWidget_rois)

    def retranslateUi(self, ControlsDock):
        _translate = QtCore.QCoreApplication.translate
        ControlsDock.setWindowTitle(_translate("ControlsDock", "Co&ntrols"))
        self.label.setText(_translate("ControlsDock", "data_column: "))
        self.comboBox_data_column.setToolTip(_translate("ControlsDock", "data column from which stimulus periods will be extracted"))
        self.checkBoxIncludeUnlabelled.setToolTip(_translate("ControlsDock", "If checked, any time periods that are not labelled with a stimulus will be considered as \"None\".\n"
"If unchecked, these unlabelled regions are ignored."))
        self.checkBoxIncludeUnlabelled.setText(_translate("ControlsDock", "Include unlabelled regions as \"None\""))
        self.label_2.setText(_translate("ControlsDock", "method: "))
        self.comboBox_method.setToolTip(_translate("ControlsDock", "method for creating the tuning curve"))
        self.label_3.setText(_translate("ControlsDock", "start offset:"))
        self.spinBox_start_offset.setToolTip(_translate("ControlsDock", "start offset"))
        self.label_4.setText(_translate("ControlsDock", "end offset:"))
        self.spinBox_end_offset.setToolTip(_translate("ControlsDock", "end offset"))
        self.label_5.setText(_translate("ControlsDock", "DPT column: "))
        self.comboBox_DPT_column.setToolTip(_translate("ControlsDock", "data to show in the datapoint tracer"))
        self.pushButton_set.setText(_translate("ControlsDock", "Set"))
        self.pushButton_save.setText(_translate("ControlsDock", "Save"))
        self.label_6.setText(_translate("ControlsDock", "Select Sample:"))
        self.label_7.setText(_translate("ControlsDock", "Select ROI:"))

from ....pyqtgraphCore.widgets.ComboBox import ComboBox
from ....pyqtgraphCore.widgets.ListWidget import ListWidget



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
@author: kushal

Chatzigeorgiou Group
Sars International Centre for Marine Molecular Biology

GNU GENERAL PUBLIC LICENSE Version 3, 29 June 2007
"""

from ..core.common import ViewerUtils
from .pytemplates.cnmf_3d_pytemplate import *
from ...common import get_window_manager
from ...common.qdialogs import *
from ...common.utils import HdfTools
from uuid import UUID
from shutil import copy
import os
from copy import deepcopy
import logging


logger = logging.getLogger()


class ModuleGUI(QtWidgets.QDockWidget):
    def __init__(self, parent, viewer_reference):
        self.vi = ViewerUtils(viewer_reference)

        QtWidgets.QDockWidget.__init__(self, parent)

        self.ui = Ui_CNMF3D_template()
        self.ui.setupUi(self)

        # If keep memmap, cannot use another item's memmap
        self.ui.checkBox_keep_memmap.toggled.connect(self.set_memmap_checkboxes)

        # If use another item's memmap, can't keep it's own memmap
        self.ui.groupBox_use_previous_memmap.toggled.connect(self.set_memmap_checkboxes)

        self.ui.pushButton_add_to_batch.clicked.connect(lambda: self.add_to_batch())

    def set_memmap_checkboxes(self):
        if self.ui.checkBox_keep_memmap.isChecked():
            self.ui.groupBox_use_previous_memmap.setChecked(False)

        elif self.ui.groupBox_use_previous_memmap.isChecked():
            self.ui.checkBox_keep_memmap.setChecked(False)

    @present_exceptions()
    def get_params(self, *args, group_params: bool = False) -> dict:
        """
        Get a dict of the params.
        Doesn't use arguments, *args and **kwargs are there just for compatibility with `@present_exceptions`
        """
        if self.vi.viewer.workEnv.imgdata.meta['fps'] == 0:
            raise KeyError('No framerate for current image sequence!',
                           'You must set a framerate for the current image sequence. '
                           'You can set it manually in the console like this:\nget_meta()["fps"] = <framerate>')

        # kwargs for CNMF
        cnmf_kwargs = {
                        'k': self.ui.spinBox_k.value(),
                        'gSig': [
                            self.ui.spinBox_gSig_x.value(),
                            self.ui.spinBox_gSig_y.value(),
                            self.ui.spinBox_gSig_z.value()
                        ],
                        'merge_thresh': self.ui.doubleSpinBox_merge_threshold.value(),
                        'p': self.ui.spinBox_p.value()
        }

        # Any additional cnmf kwargs
        if self.ui.groupBox_cnmf_kwargs.isChecked():
            try:
                _kwargs = self.ui.plainTextEdit_cnmf_kwargs.toPlainText()
                cnmf_kwargs_add = eval(f"dict({_kwargs})")
                cnmf_kwargs.update(cnmf_kwargs_add)
            except:
                raise ValueError("CNMF kwargs not formatted properly.")


        # kwargs for component evaluation
        eval_kwargs = {
                        'fr':           self.vi.viewer.workEnv.imgdata.meta['fps'],
                        'decay_time':   self.ui.doubleSpinBox_decay_time.value(),
                        'min_SNR':      self.ui.doubleSpinBox_minSNR.value(),
                        'rval_thr':     self.ui.doubleSpinBox_rval.value(),
        }

        # Any additional eval kwargs
        if self.ui.groupBox_eval_kwargs.isChecked():
            try:
                _kwargs = self.ui.plainTextEdit_eval_kwargs.toPlainText()
                eval_kwargs_add = eval(f"dict{_kwargs}")
                eval_kwargs.update(eval_kwargs_add )
            except:
                raise ValueError("Evaluation kwargs not formatted properly.")

        use_memmap = self.ui.groupBox_use_previous_memmap.isChecked()
        memmap_uuid = self.ui.lineEdit_memmap_uuid.text()

        if use_memmap:
            try:
                UUID(memmap_uuid)
            except ValueError:
                raise ValueError('Invalid UUID format. Please check that you entered the UUID correctly.')

        d = {
            'refit':        self.ui.checkBox_refit.isChecked(),
            'item_name':    self.ui.lineEdit_name.text(),
            'use_patches':  self.ui.groupBox_use_patches.isChecked(),
            'use_memmap':   use_memmap,
            'memmap_uuid':  memmap_uuid,
            'keep_memmap':  self.ui.checkBox_keep_memmap.isChecked()
        }

        if d['use_patches']:
            cnmf_kwargs.update(
                {
                    'rf': self.ui.spinBox_rf.value(),
                    'stride': self.ui.spinBox_stride.value(),
                }
            )

        if group_params:
            d.update(
                {
                    'cnmf_kwargs': cnmf_kwargs,
                    'eval_kwargs': eval_kwargs,
                }
            )
        else:
            d.update(
                {
                    **cnmf_kwargs,
                    **eval_kwargs,
                }
            )

        return d

    @present_exceptions()
    def add_to_batch(self, params: dict = None) -> UUID:
        if self.vi.viewer.workEnv.isEmpty:
            QtWidgets.QMessageBox.warning(self, 'Empty work environment', 'The work environment is empty, '
                                                                          'nothing to add to batch')
            return

        if params is None:
            d = self.get_params(group_params=True)
        else:
            # Check that user passed dict is formatted correctly
            required_keys = ['cnmf_kwargs', 'eval_kwargs', 'item_name', 'refit', 'use_patches',
                             'use_memmap', 'memmap_uuid', 'keep_memmap']
            if any([k not in params.keys() for k in required_keys]):
                raise ValueError(f'Must pass a params dict with the following keys:\n'
                                 f'{required_keys}\n'
                                 f'Please see the docs for more information.')
            d = deepcopy(params)

        name = d['item_name']

        self.vi.viewer.status_bar_label.showMessage(f'Please wait, adding CNMF 3D item: "{name}" to batch')

        bm = get_window_manager().get_batch_manager()

        if d['use_memmap']:
            memmap_uuid = UUID(d['memmap_uuid'])

            if memmap_uuid not in bm.df['uuid'].values:
                raise ValueError("The entered memmap UUID isn't present in this batch")

            bm.df.loc[bm.df['uuid'] == memmap_uuid, 'save_temp_files'] = 1

        if self.ui.groupBox_seed_components.isChecked():
            seed_path = self.ui.lineEdit_seed_components_path.text()
            if not os.path.isfile(seed_path):
                raise FileNotFoundError(
                    "Seed file does not exist, check the path"
                )

            seed_params = HdfTools.load_dict(seed_path, 'data/segment_params')

            d['use_seeds'] = True
            d['seed_params'] = seed_params

        else:
            d['use_seeds'] = False

        u = bm.add_item(
            module='CNMF_3D',
            name=name,
            input_workEnv=self.vi.viewer.workEnv,
            input_params=d,
            info=d
        )

        if u is None:
            self.vi.viewer.status_bar_label.clearMessage()
            return

        if d['use_seeds']:
            logger.info("Copying component seeds files")
            copy(seed_path, os.path.join(bm.batch_path, f'{u}.ain'))

        if d['keep_memmap']:
            bm.df.loc[bm.df['uuid'] == u, 'save_temp_files'] = 1

        self.vi.viewer.status_bar_label.showMessage(f'Finished adding CNMF 3D item: "{name}" to batch')
        self.ui.lineEdit_name.clear()
        self.ui.lineEdit_memmap_uuid.clear()

        return u



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on July 2, 2018

@author: kushal

Chatzigeorgiou Group
Sars International Centre for Marine Molecular Biology

GNU GENERAL PUBLIC LICENSE Version 3, 29 June 2007

"""

from PyQt5 import QtCore, QtGui, QtWidgets
# from pyqtgraphCore.widgets.ColorButton import ColorButton
import pandas as pd
from ...common import configuration, get_project_manager
from .stimmap_modules.page import Page
from .stimmap_modules.row import Row, ColorButton
from .stimulus_mapping import ModuleGUI# as StimulusMappingModuleGUI


class MesStimmapGUI(ModuleGUI):
    def __init__(self, parent, viewer):
        super(MesStimmapGUI, self).__init__(parent, viewer)
        self.ui.comboBoxShowTimelineChoice.setVisible(False)
        self.ui.label.setVisible(False)
        self.clear_all_tabs()
        self.ui.btnSetAllMaps.clicked.connect(self.set_all_voltage_mappings)
        self.voltage_mappings = None

    def add_stim_type(self, stim_type: str):
        self.ui.tabWidget.addTab(PageMes(parent=self, stim_type=stim_type), stim_type)
        self.tabs[stim_type] = self.ui.tabWidget.widget(self.ui.tabWidget.count() - 1)

    def get_all_stims_dataframes(self) -> dict:
        d = {}

        for ix in range(self.ui.tabWidget.count()):
            w = self.ui.tabWidget.widget(ix)
            assert isinstance(w, PageMes)
            stim_def = w.combobox_stim_defs.currentText()
            if stim_def != '':
                d[stim_def] = {'channel': self.ui.tabWidget.tabText(ix),
                               'dataframe': w.get_dataframe()}

        return d

    def set_all_voltage_mappings(self):
        self.voltage_mappings = self.get_all_stims_dataframes()

    def export_to_work_env(self):
        pass


class PageMes(Page):
    def __init__(self, parent, stim_type: str):
        super(PageMes, self).__init__(parent, stim_type)
        self.ui.comboBoxTimeUnits.setVisible(False)
        self.ui.btnAddRow.setVisible(False)
        self.ui.label.setVisible(False)

        label_stim_def = QtWidgets.QLabel()
        label_stim_def.setText('Choose Stimulus Type (from project config):')
        self.ui.verticalLayout.insertWidget(0, label_stim_def)

        self.combobox_stim_defs = QtWidgets.QComboBox()
        self.update_stim_defs_combobox()
        get_project_manager().signal_dataframe_changed.connect(self.update_stim_defs_combobox)
        self.ui.verticalLayout.insertWidget(1, self.combobox_stim_defs)

    def update_stim_defs_combobox(self):
        self.combobox_stim_defs.clear()
        self.combobox_stim_defs.addItems([''])
        stim_defs = configuration.proj_cfg.options('STIM_DEFS')
        stim_defs.sort()
        self.combobox_stim_defs.addItems(stim_defs)

    def add_row(self, pd_series=None):
        """
        :type pd_series: pd.Series
        :type pd_series: pd.Series
        """
        row = RowMes(pd_series)
        self.ui.verticalLayout.insertLayout(self.ui.verticalLayout.count() - 1, row.hlayout)
        self.rows.append(row)


class RowMes(QtWidgets.QWidget):
    def __init__(self, pd_series: pd.Series):
        QtWidgets.QWidget.__init__(self)

        self.hlayout = QtWidgets.QHBoxLayout()

        self.voltage_label = QtWidgets.QLabel(self)
        self.hlayout.addWidget(self.voltage_label)

        self.name = QtWidgets.QLineEdit(self)
        self.name.setPlaceholderText('Stimulus Name')
        self.hlayout.addWidget(self.name)

        self.color_btn = ColorButton(self)
        self.hlayout.addWidget(self.color_btn)

        if pd_series is not None:
            self.name.setText(pd_series['name'])
            self.color_btn.setColor(pd_series['color'])
            self.voltage_label.setText(pd_series['voltage'])

    def set_series(self, pd_series: pd.Series):
        self.name.setText(pd_series['name'].item())
        self.color_btn.setColor(pd_series['color'].item())
        self.voltage_label.setText(pd_series['voltage'].item())

    def get_dict(self) -> dict:
        d = {'voltage': self.voltage_label.text(),
             'name':    self.name.text(),
             'color':   self.color_btn.color()
             }
        return d

    # def delete(self):
    #     self.qlabel.deleteLater()
    #     self.name.deleteLater()
    #     self.start.deleteLater()
    #     self.end.deleteLater()
    #     self.btn_remove.deleteLater()
    #     self.color_btn.deleteLater()

if __name__ == '__main__':
    app = QtWidgets.QApplication([])
    w = ModuleGUI(parent=None, viewer=None)
    w.add_stim_type('bah test')
    w.add_stim_type('bah test2')
    w.show()
    app.exec_()


# -*- coding: utf-8 -*-
"""
Created on July 7 2018

@author: kushal

Chatzigeorgiou Group
Sars International Centre for Marine Molecular Biology

GNU GENERAL PUBLIC LICENSE Version 3, 29 June 2007
"""


import numpy as np
from scipy.fftpack import rfft, irfft
import sys


def interpolate_irfft(raw_curve: np.ndarray, rfft_curve: np.ndarray, stop_domain: int) -> np.ndarray:
    """
    1D Interpolate of the inverse fourier transform curve so that it is of the same length as the raw curve and can
    be used to calculate the sum of squared differences between them.

    :param raw_curve:   Raw curve
    :param rfft_curve:  Result from scipy.fftpack.rfft(raw_curve)
    :param stop_domain: Number of fourier domains to include for computing the inverse fourier transform
    :return:
    """
    x = np.arange(0, len(raw_curve))
    irf = irfft(rfft_curve[:stop_domain])
    xp = np.linspace(0, raw_curve.size, stop_domain)
    interp = np.interp(x, xp, fp=irf)
    return interp


def sum_of_squared_diff(a: np.ndarray, b: np.ndarray):
    """
    Calculate the sum of squared differences between "a" and "b"
    a and b must be of the same size
    :param a:   1D vector, [x1, x2, x3, ... xn]
    :param b:   1D vector, [x1, x2, x3, ... xn]
    :return:    Sum of squared differences between a and b
    """
    if a.size != b.size:
        raise ValueError('"a" and "b" must be of the same size.')
    r = a - b
    return np.dot(r, r)


def get_residuals(c: np.ndarray) -> np.ndarray:
    """
    :param c:   1D array representing a single curve
    :return:    residuals between the c and irfft of c with increasing steps of frequency domains
    """
    c_r = rfft(c)
    r = np.zeros(shape=(c.size - 1), dtype=np.float64)
    for step in range(1, c.size):
        r[step - 1] = sum_of_squared_diff(c, interpolate_irfft(raw_curve=c, rfft_curve=c_r, stop_domain=step))
    return r


def get_all_residuals(a: np.ndarray) -> np.ndarray:
    """
    :param a:   2D array of 1D curves
    :return:    2D array of residuals for each curve
    """
    r = np.zeros(shape=(a.shape[0], a.shape[1] - 1), dtype=np.float64)
    for i in range(a.shape[0]):
        r[i, :] = get_residuals(a[i, :])
        sys.stdout.write(f"\rProgress: {i/(a.shape[0] - 1)}")
    return r



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on June 17 2018

@author: kushal

Chatzigeorgiou Group
Sars International Centre for Marine Molecular Biology

GNU GENERAL PUBLIC LICENSE Version 3, 29 June 2007
"""

from PyQt5 import QtCore, QtGui, QtWidgets
from .tab_area_widget import TabAreaWidget
import pandas as pd
from ...common import configuration, get_project_manager, get_window_manager
from ...misc_widgets.list_widget_dialog import ListWidgetDialog
from functools import partial
from ...viewer.core.common import ViewerUtils
from ...viewer.core.viewer_work_environment import ViewerWorkEnv
from collections import UserList
# from ...common.window_manager import WindowClass
from ...common import start
import os


class ProjectBrowserWidget(QtWidgets.QWidget):
    def __init__(self, parent, dataframe: pd.DataFrame):
        QtWidgets.QWidget.__init__(self, parent=parent)

        self.tabs = {}
        self.dataframe = dataframe

        self.vertical_layout = QtWidgets.QVBoxLayout(self)

        self.tab_widget = QtWidgets.QTabWidget(parent=self)

        # self.tabs_widget.tabBar().tabCloseRequested.connect(lambda ix: self.del_tab(ix))

        self.vertical_layout.addWidget(self.tab_widget)

        self.add_tab(dataframe, [], is_root=True)

        self.tab_widget.tabCloseRequested.connect(lambda ix: self.del_tab(ix))

        get_project_manager().signal_dataframe_changed.connect(self.update_dataframe_data)

    @QtCore.pyqtSlot(pd.DataFrame)
    def update_dataframe_data(self, dataframe: pd.DataFrame):
        self.dataframe = dataframe  # This is probably not necessary

        tab_area = self.tabs['root']
        assert isinstance(tab_area, TabAreaWidget)

        tab_area.dataframe = self.dataframe
        tab_area.populate_tab()

        for tab_name in self.tabs.keys():
            pass

    def _create_root_context_menu(self):
        pass

    def add_tab(self, dataframe: pd.DataFrame, filter_history: list, is_root=False, name: str = None):
        if not is_root and name is None:
            tab_name = QtWidgets.QInputDialog.getText(self, None, 'Enter name for new tab: ')
            if tab_name[0] == '' or tab_name[1] is False:
                return
            elif tab_name[0] in get_project_manager().child_dataframes.keys():
                QtWidgets.QMessageBox.warning(self, 'DataFrame title already exists!',
                                              'That name already exists in your project, choose a different name!')
                self.add_tab(dataframe, filter_history, is_root)
            tab_name = tab_name[0]
            get_project_manager().add_sub_dataframe(tab_name, filter_history, dataframe)
        elif is_root:
            tab_name = 'root'
        else:
            tab_name = name

        tab_area_widget = TabAreaWidget(self.tab_widget, tab_name, dataframe, filter_history, is_root)
        tab_area_widget.signal_new_tab_requested.connect(self.slot_new_tab_requested)
        tab_area_widget.signal_open_sample_in_viewer_requested.connect(self.slot_open_sample_id_in_viewer)

        self.tab_widget.addTab(tab_area_widget, tab_name)

        self.tab_widget.setTabsClosable(True)
        bar = self.tab_widget.tabBar()
        bar.setTabButton(0, bar.RightSide, None)

        self.tabs.update({tab_name: tab_area_widget})

        if is_root:
            num_columns = len(tab_area_widget.columns)
            self.resize(min(1920, num_columns * 215), 400)

        self.tab_widget.setCurrentIndex(self.tab_widget.count() - 1)

    @QtCore.pyqtSlot(pd.DataFrame, list)
    def slot_new_tab_requested(self, dataframe: pd.DataFrame, filter_history: list):
        self.add_tab(dataframe, filter_history)

    def del_tab(self, ix: int):
        print(ix)
        if QtWidgets.QMessageBox.question(self, 'Remove Tab?', 'Are you sure you want to delete this tab? '
                                                       'Only the filter operations to get this child DataFrame will be '
                                                       'removed, your data is still in the Root DataFrame.',
                                      QtWidgets.QMessageBox.Yes, QtWidgets.QMessageBox.No) == QtWidgets.QMessageBox.No:
            return

        tab_name = self.tab_widget.widget(ix).tab_name
        self.tab_widget.removeTab(ix)
        get_project_manager().remove_sub_dataframe(tab_name)

    @QtCore.pyqtSlot(str)
    def slot_open_sample_id_in_viewer(self, sample_id: str):
        viewer = get_window_manager().get_new_viewer_window().viewer_reference

        vi = ViewerUtils(viewer_reference=viewer)

        row = self.dataframe[self.dataframe['SampleID'] == sample_id].iloc[0]
        pikPath = os.path.join(configuration.proj_path, row['ImgInfoPath'])
        tiffPath = os.path.join(configuration.proj_path, row['ImgPath'])

        vi.viewer.workEnv = ViewerWorkEnv.from_pickle(pickle_file_path=pikPath, tiff_path=tiffPath)
        vi.update_workEnv()
        vi.viewer.workEnv.restore_rois_from_states()
        vi.viewer.ui.label_curr_img_seq_name.setText(sample_id)

        try:
            self.lwd.deleteLater()
        except:
            pass

        vi.viewer.workEnv.saved = True
        


# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'tiff_io_pytemplate.ui'
#
# Created by: PyQt5 UI code generator 5.9.2
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets

class Ui_DockWidget(object):
    def setupUi(self, DockWidget):
        DockWidget.setObjectName("DockWidget")
        DockWidget.resize(600, 411)
        DockWidget.setFloating(True)
        self.dockWidgetContents = QtWidgets.QWidget()
        self.dockWidgetContents.setObjectName("dockWidgetContents")
        self.verticalLayout_6 = QtWidgets.QVBoxLayout(self.dockWidgetContents)
        self.verticalLayout_6.setObjectName("verticalLayout_6")
        self.horizontalLayout_4 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_4.setObjectName("horizontalLayout_4")
        self.verticalLayout_4 = QtWidgets.QVBoxLayout()
        self.verticalLayout_4.setObjectName("verticalLayout_4")
        self.label = QtWidgets.QLabel(self.dockWidgetContents)
        font = QtGui.QFont()
        font.setPointSize(14)
        font.setBold(True)
        font.setWeight(75)
        self.label.setFont(font)
        self.label.setObjectName("label")
        self.verticalLayout_4.addWidget(self.label)
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.btnSelectTiff = QtWidgets.QPushButton(self.dockWidgetContents)
        self.btnSelectTiff.setMaximumSize(QtCore.QSize(90, 32))
        self.btnSelectTiff.setObjectName("btnSelectTiff")
        self.horizontalLayout.addWidget(self.btnSelectTiff)
        self.labelFileTiff = QtWidgets.QLabel(self.dockWidgetContents)
        self.labelFileTiff.setText("")
        self.labelFileTiff.setTextInteractionFlags(QtCore.Qt.LinksAccessibleByMouse|QtCore.Qt.TextSelectableByKeyboard|QtCore.Qt.TextSelectableByMouse)
        self.labelFileTiff.setObjectName("labelFileTiff")
        self.horizontalLayout.addWidget(self.labelFileTiff)
        self.verticalLayout_4.addLayout(self.horizontalLayout)
        self.groupBox = QtWidgets.QGroupBox(self.dockWidgetContents)
        self.groupBox.setObjectName("groupBox")
        self.verticalLayout_2 = QtWidgets.QVBoxLayout(self.groupBox)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.radioButtonAsArray = QtWidgets.QRadioButton(self.groupBox)
        self.radioButtonAsArray.setObjectName("radioButtonAsArray")
        self.verticalLayout_2.addWidget(self.radioButtonAsArray)
        self.radioButtonAsArrayMulti = QtWidgets.QRadioButton(self.groupBox)
        self.radioButtonAsArrayMulti.setObjectName("radioButtonAsArrayMulti")
        self.verticalLayout_2.addWidget(self.radioButtonAsArrayMulti)
        self.radioButtonImread = QtWidgets.QRadioButton(self.groupBox)
        self.radioButtonImread.setObjectName("radioButtonImread")
        self.verticalLayout_2.addWidget(self.radioButtonImread)
        self.verticalLayout_4.addWidget(self.groupBox)
        self.groupBox_2 = QtWidgets.QGroupBox(self.dockWidgetContents)
        self.groupBox_2.setObjectName("groupBox_2")
        self.verticalLayout = QtWidgets.QVBoxLayout(self.groupBox_2)
        self.verticalLayout.setObjectName("verticalLayout")
        self.radioButton_axes_default = QtWidgets.QRadioButton(self.groupBox_2)
        self.radioButton_axes_default.setChecked(True)
        self.radioButton_axes_default.setObjectName("radioButton_axes_default")
        self.verticalLayout.addWidget(self.radioButton_axes_default)
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.radioButton_axes_custom = QtWidgets.QRadioButton(self.groupBox_2)
        self.radioButton_axes_custom.setText("")
        self.radioButton_axes_custom.setObjectName("radioButton_axes_custom")
        self.horizontalLayout_2.addWidget(self.radioButton_axes_custom)
        self.lineEdit_axes_custom = QtWidgets.QLineEdit(self.groupBox_2)
        self.lineEdit_axes_custom.setText("")
        self.lineEdit_axes_custom.setMaxLength(4)
        self.lineEdit_axes_custom.setObjectName("lineEdit_axes_custom")
        self.horizontalLayout_2.addWidget(self.lineEdit_axes_custom)
        self.verticalLayout.addLayout(self.horizontalLayout_2)
        self.verticalLayout_4.addWidget(self.groupBox_2)
        self.horizontalLayout_4.addLayout(self.verticalLayout_4)
        self.line = QtWidgets.QFrame(self.dockWidgetContents)
        self.line.setFrameShape(QtWidgets.QFrame.VLine)
        self.line.setFrameShadow(QtWidgets.QFrame.Sunken)
        self.line.setObjectName("line")
        self.horizontalLayout_4.addWidget(self.line)
        self.verticalLayout_5 = QtWidgets.QVBoxLayout()
        self.verticalLayout_5.setObjectName("verticalLayout_5")
        self.label_2 = QtWidgets.QLabel(self.dockWidgetContents)
        font = QtGui.QFont()
        font.setPointSize(14)
        font.setBold(True)
        font.setWeight(75)
        self.label_2.setFont(font)
        self.label_2.setObjectName("label_2")
        self.verticalLayout_5.addWidget(self.label_2)
        self.groupBox_meta_loader = QtWidgets.QGroupBox(self.dockWidgetContents)
        self.groupBox_meta_loader.setCheckable(True)
        self.groupBox_meta_loader.setObjectName("groupBox_meta_loader")
        self.verticalLayout_3 = QtWidgets.QVBoxLayout(self.groupBox_meta_loader)
        self.verticalLayout_3.setObjectName("verticalLayout_3")
        self.horizontalLayout_3 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_3.setObjectName("horizontalLayout_3")
        self.btnSelectMetaFile = QtWidgets.QPushButton(self.groupBox_meta_loader)
        self.btnSelectMetaFile.setMaximumSize(QtCore.QSize(130, 32))
        self.btnSelectMetaFile.setObjectName("btnSelectMetaFile")
        self.horizontalLayout_3.addWidget(self.btnSelectMetaFile)
        self.labelFileMeta = QtWidgets.QLabel(self.groupBox_meta_loader)
        self.labelFileMeta.setText("")
        self.labelFileMeta.setTextInteractionFlags(QtCore.Qt.LinksAccessibleByMouse|QtCore.Qt.TextSelectableByKeyboard|QtCore.Qt.TextSelectableByMouse)
        self.labelFileMeta.setObjectName("labelFileMeta")
        self.horizontalLayout_3.addWidget(self.labelFileMeta)
        self.verticalLayout_3.addLayout(self.horizontalLayout_3)
        self.label_3 = QtWidgets.QLabel(self.groupBox_meta_loader)
        self.label_3.setObjectName("label_3")
        self.verticalLayout_3.addWidget(self.label_3)
        self.listWidget_meta_data_loader = QtWidgets.QListWidget(self.groupBox_meta_loader)
        self.listWidget_meta_data_loader.setObjectName("listWidget_meta_data_loader")
        self.verticalLayout_3.addWidget(self.listWidget_meta_data_loader)
        self.verticalLayout_5.addWidget(self.groupBox_meta_loader)
        self.horizontalLayout_4.addLayout(self.verticalLayout_5)
        self.verticalLayout_6.addLayout(self.horizontalLayout_4)
        self.btnLoadIntoWorkEnv = QtWidgets.QPushButton(self.dockWidgetContents)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.MinimumExpanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.btnLoadIntoWorkEnv.sizePolicy().hasHeightForWidth())
        self.btnLoadIntoWorkEnv.setSizePolicy(sizePolicy)
        self.btnLoadIntoWorkEnv.setObjectName("btnLoadIntoWorkEnv")
        self.verticalLayout_6.addWidget(self.btnLoadIntoWorkEnv)
        DockWidget.setWidget(self.dockWidgetContents)

        self.retranslateUi(DockWidget)
        QtCore.QMetaObject.connectSlotsByName(DockWidget)
        DockWidget.setTabOrder(self.btnSelectTiff, self.radioButtonAsArray)
        DockWidget.setTabOrder(self.radioButtonAsArray, self.radioButtonAsArrayMulti)
        DockWidget.setTabOrder(self.radioButtonAsArrayMulti, self.radioButtonImread)
        DockWidget.setTabOrder(self.radioButtonImread, self.radioButton_axes_default)
        DockWidget.setTabOrder(self.radioButton_axes_default, self.radioButton_axes_custom)
        DockWidget.setTabOrder(self.radioButton_axes_custom, self.lineEdit_axes_custom)
        DockWidget.setTabOrder(self.lineEdit_axes_custom, self.groupBox_meta_loader)
        DockWidget.setTabOrder(self.groupBox_meta_loader, self.btnSelectMetaFile)
        DockWidget.setTabOrder(self.btnSelectMetaFile, self.listWidget_meta_data_loader)
        DockWidget.setTabOrder(self.listWidget_meta_data_loader, self.btnLoadIntoWorkEnv)

    def retranslateUi(self, DockWidget):
        _translate = QtCore.QCoreApplication.translate
        DockWidget.setWindowTitle(_translate("DockWidget", "Tiff file &I/O"))
        self.label.setText(_translate("DockWidget", "Tiff File"))
        self.btnSelectTiff.setText(_translate("DockWidget", "Select tiff file"))
        self.groupBox.setToolTip(_translate("DockWidget", "The options for “Load Method” correspond to the tifffile library method that is used for loading the images"))
        self.groupBox.setTitle(_translate("DockWidget", "Image Load Method:"))
        self.radioButtonAsArray.setToolTip(_translate("DockWidget", "Usually faster alternative to imread"))
        self.radioButtonAsArray.setText(_translate("DockWidget", "asarray"))
        self.radioButtonAsArrayMulti.setToolTip(_translate("DockWidget", "Use if tiff was created in append mode"))
        self.radioButtonAsArrayMulti.setText(_translate("DockWidget", "asarray - multi series"))
        self.radioButtonImread.setToolTip(_translate("DockWidget", "Should work for most tiffs if they were not created in append mode."))
        self.radioButtonImread.setText(_translate("DockWidget", "i&mread"))
        self.groupBox_2.setTitle(_translate("DockWidget", "Image Axes order"))
        self.radioButton_axes_default.setToolTip(_translate("DockWidget", "default option, usual dimensions"))
        self.radioButton_axes_default.setText(_translate("DockWidget", "&2D: [t, x, y]\n"
"3D: [t, z, x, y]"))
        self.lineEdit_axes_custom.setPlaceholderText(_translate("DockWidget", "Custom, ex: xyzt"))
        self.label_2.setText(_translate("DockWidget", "Meta data file"))
        self.groupBox_meta_loader.setToolTip(_translate("DockWidget", "Select a meta data loader that corresonds to your meta data file.\n"
"See the docs for instructions on how to create your own loader."))
        self.groupBox_meta_loader.setTitle(_translate("DockWidget", "&Load meta data"))
        self.btnSelectMetaFile.setText(_translate("DockWidget", "Select file"))
        self.label_3.setText(_translate("DockWidget", "Meta data format:"))
        self.btnLoadIntoWorkEnv.setText(_translate("DockWidget", "Load into Work Environment"))




#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Sun Feb 2 2017

@author: kushal

Chatzigeorgiou Group
Sars International Centre for Marine Molecular Biology

GNU GENERAL PUBLIC LICENSE Version 3, 29 June 2007

"""
import sys
from . import get_project_manager, get_window_manager
from . import configuration
from .pytemplates.project_config_pytemplate import *
from numpy import int64, float64
from glob import glob
import pickle


'''
Just a simple GUI for modifying the project config file.
'''


class ColumnsPage(QtWidgets.QWidget):
    def __init__(self, parent=None, *args):
        QtWidgets.QWidget.__init__(self, parent, *args)
        self._context_menu_selected_list_widget = None
        self.setWindowTitle('Project Configuration')

    def setupGUI(self):
        self.ui = Ui_Form()
        self.ui.setupUi(self)
        configuration.open_proj_config()
        self.ui.listwInclude.addItems(configuration.proj_cfg.options('INCLUDE'))
        self.ui.listwExclude.addItems(configuration.proj_cfg.options('EXCLUDE'))
        self.ui.listwROIDefs.addItems(configuration.proj_cfg.options('ROI_DEFS'))
        self.ui.listwStimDefs.addItems(configuration.proj_cfg.options('STIM_DEFS'))
        self.ui.listwCustomColumns.addItems(configuration.proj_cfg.options('CUSTOM_COLUMNS'))

        self.ui.radioButtonBoolean.clicked.connect(self.disable_custom_column_line_edit_validator)
        self.ui.radioButtonStr.clicked.connect(self.disable_custom_column_line_edit_validator)
        self.ui.radioButtonInt64.clicked.connect(self.set_custom_column_line_edit_validator)
        self.ui.radioButtonFloat64.clicked.connect(self.set_custom_column_line_edit_validator)
        
        self.ui.btnAddNewROICol.clicked.connect(self.add_roi_def)
        self.ui.btnAddNewStimCol.clicked.connect(self.add_stim_def)

        self.ui.btnAddCustomColumn.clicked.connect(self.add_custom_column)

        self.ui.btnSave.clicked.connect(self.save_config)

        self.custom_to_add = {}

        action_delete_column = QtWidgets.QWidgetAction(self)
        action_delete_column.setText('Delete column')
        action_delete_column.triggered.connect(self.delete_column)

        self.delete_column_menu = QtWidgets.QMenu(self)
        self.delete_column_menu.addAction(action_delete_column)

        self.ui.listwROIDefs.setContextMenuPolicy(QtCore.Qt.CustomContextMenu)
        self.ui.listwROIDefs.customContextMenuRequested.connect(lambda p: self.bulid_delete_menu(p, self.ui.listwROIDefs))

        self.ui.listwStimDefs.setContextMenuPolicy(QtCore.Qt.CustomContextMenu)
        self.ui.listwStimDefs.customContextMenuRequested.connect(lambda p: self.bulid_delete_menu(p, self.ui.listwStimDefs))

        self.ui.listwCustomColumns.setContextMenuPolicy(QtCore.Qt.CustomContextMenu)
        self.ui.listwCustomColumns.customContextMenuRequested.connect(lambda p: self.bulid_delete_menu(p, self.ui.listwCustomColumns))

        self.new_roi_defs = []
        # TODO: Deletable columns for ROI, Stim, and Custom columns.

    def bulid_delete_menu(self, p, list_widget):
        self._context_menu_selected_list_widget = list_widget
        self.delete_column_menu.exec_(list_widget.mapToGlobal(p))

    def delete_column(self):
        if self._context_menu_selected_list_widget is None:
            return
        item = self._context_menu_selected_list_widget.takeItem(self._context_menu_selected_list_widget.currentRow())

        includes = [self.ui.listwInclude.item(i).text() for i in range(self.ui.listwInclude.count())]

        if item.text() in includes:
            self.ui.listwInclude.takeItem(includes.index(item.text()))
            return

        excludes = [self.ui.listwExclude.item(i).text() for i in range(self.ui.listwExclude.count())]
        if item.text() in excludes:
            self.ui.listwExclude.takeItem(excludes.index(item.text()))

    def add_roi_def(self):
        if self.ui.lineEdNewROIDef.text() != '':
            text = self.ui.lineEdNewROIDef.text().replace(' ', '_')



            if text in get_project_manager().dataframe.columns:
                QtWidgets.QMessageBox.warning(self, 'Name already exists',
                                              'The entered column name already exists in the dataframe. '
                                              'Enter a different name.')
                return

            self.ui.listwROIDefs.addItem(text)
            self.ui.listwInclude.addItem(text)
            self.ui.lineEdNewROIDef.clear()
            self.new_roi_defs.append(text)
        
    def add_stim_def(self):
        if self.ui.lineEdNewStimCol.text() != '':
            text = self.ui.lineEdNewStimCol.text().replace(' ', '_')

            if text in get_project_manager().dataframe.columns:
                QtWidgets.QMessageBox.warning(self, 'Name already exists',
                                              'The entered column name already exists in the dataframe. '
                                              'Enter a different name.')

            self.ui.listwStimDefs.addItem(text)
            self.ui.listwInclude.addItem(text)
            self.ui.lineEdNewStimCol.clear()

    def save_config(self):
        if len(get_window_manager().flowcharts) > 0:
            if QtWidgets.QMessageBox.question(self,
                                              'Flowcharts are open',
                                              'Updating the project configuration while you are actively using a flowchart '
                                              'will remove the inputs to any <Load_Proj_DF> that are open and you will '
                                              'need to create these nodes again.\n'
                                              'Do you still want to continue?',
                                               QtWidgets.QMessageBox.Yes,
                                               QtWidgets.QMessageBox.No) == QtWidgets.QMessageBox.No:
                return

        include = []
        for i in range(0, self.ui.listwInclude.count()):
            include.append(self.ui.listwInclude.item(i).text())
        configuration.proj_cfg['INCLUDE'] = dict.fromkeys(include)
        
        exclude = []
        for i in range(0, self.ui.listwExclude.count()):
            exclude.append(self.ui.listwExclude.item(i).text())
        configuration.proj_cfg['EXCLUDE'] = dict.fromkeys(exclude)
        
        roi_defs = []
        for i in range(0, self.ui.listwROIDefs.count()):
            roi_defs.append(self.ui.listwROIDefs.item(i).text())
        configuration.proj_cfg['ROI_DEFS'] = dict.fromkeys(roi_defs)

        images_dir = configuration.proj_path + '/images'
        for new_def in self.new_roi_defs:
            for f in glob(images_dir + '/*.pik'):
                p = pickle.load(open(f, 'rb'))
                for s in p['roi_states']['states']:
                    s['tags'].update({new_def: 'untagged'})
                pickle.dump(p, open(f, 'wb'))

        self.new_roi_defs = []

        stim_defs = []
        for i in range(0, self.ui.listwStimDefs.count()):
            stim_defs.append(self.ui.listwStimDefs.item(i).text())
        configuration.proj_cfg['STIM_DEFS'] = dict.fromkeys(stim_defs)

        custom_colums = [self.ui.listwCustomColumns.item(i).text() for i in range(self.ui.listwCustomColumns.count())]
        for c in configuration.proj_cfg.options('CUSTOM_COLUMNS'):
            if c not in custom_colums:
                configuration.proj_cfg.remove_option('CUSTOM_COLUMNS', c)

        for column in self.custom_to_add.keys():
            configuration.proj_cfg.set('CUSTOM_COLUMNS', column, str(self.custom_to_add[column]['type']))

        configuration.save_proj_config()
        get_project_manager().update_project_config_requested(self.custom_to_add)
        self.ui.btnClose.click()

    def set_custom_column_line_edit_validator(self):
        if self.ui.radioButtonInt64.isChecked():
            self.ui.lineEditCustomColumnReplacementValue.setValidator(QtGui.QIntValidator())
        elif self.ui.radioButtonFloat64.isChecked():
            self.ui.lineEditCustomColumnReplacementValue.setValidator(QtGui.QDoubleValidator())

    def disable_custom_column_line_edit_validator(self):
        self.ui.lineEditCustomColumnReplacementValue.setValidator(None)

    def add_custom_column(self):
        name = self.ui.lineEditCustomColumnName.text()
        if name == '':
            return

        if name in get_project_manager().dataframe.columns:
            QtWidgets.QMessageBox.warning(self, 'Name already exists',
                                          'The entered column name already exists in the dataframe. '
                                          'Enter a different name.')
            return

        replacement_value = self.ui.lineEditCustomColumnReplacementValue.text()

        if self.ui.radioButtonStr.isChecked():
            if replacement_value == '':
                replacement_value = 'untagged'
            column_type = 'str'

        elif self.ui.radioButtonInt64.isChecked():
            if replacement_value == '':
                replacement_value = 0
            else:
                replacement_value = int64(replacement_value)
            column_type = 'int64'

        elif self.ui.radioButtonFloat64.isChecked():
            if replacement_value == '':
                replacement_value = 0.0
            else:
                replacement_value = float64(replacement_value)
            column_type = 'float64'

        elif self.ui.radioButtonBoolean.isChecked():
            column_type = 'bool'

            if replacement_value not in ['True', 'False'] and not get_project_manager().dataframe.empty:
                QtWidgets.QMessageBox.warning(self, 'Invalid replacement value',
                                              'You can enter only True or False for boolean data types')
            else:
                replacement_value = bool(replacement_value)

        else:
            QtWidgets.QMessageBox.warning(self, 'No data type selected',
                                          'You must select a data type')
            return

        name = name.replace(' ', '_')
        self.ui.listwCustomColumns.addItem(name)
        self.ui.listwInclude.addItem(name)
        self.ui.lineEditCustomColumnReplacementValue.clear()
        self.ui.lineEditCustomColumnName.clear()

        self.custom_to_add.update({name: {'type': column_type, 'replacement_value': replacement_value}})


class Window(QtWidgets.QWidget):
    def __init__(self):
        super().__init__()
        self.tabs = QtWidgets.QTabWidget()

        layout = QtWidgets.QVBoxLayout(self)
        layout.addWidget(self.tabs)
        
        # Adds a new tab, places an instance of TabPage widget which is displayed in the whole tab.
        self.tabs.addTab(ColumnsPage(self.tabs), 'Columns')
        # Setup the GUI in that tab
        self.tabs.widget(self.tabs.count() - 1).setupGUI()

        self.tabs.widget(0).ui.btnClose.clicked.connect(self.close)



"""
Pandas extensions for working with Batch Manager data
"""

import pandas as pd
import pathlib
from ..common import configuration
from typing import *
from uuid import UUID
import pickle
if configuration.HAS_CAIMAN:
    import caiman as cm

CURRENT_BATCH: pathlib.Path = None  # only one batch at a time for now


def load_batch(path: Union[str, pathlib.Path]) -> pd.DataFrame:
    global CURRENT_BATCH

    df = pd.read_pickle(
        pathlib.Path(path).joinpath('/dataframe.batch')
    )

    CURRENT_BATCH = pathlib.Path(path)

    return df


def _get_item_uuid(item: Union[int, str, UUID]) -> UUID:
    pass


@pd.api.extensions.register_series_accessor("caiman")
class CaimanExtensions:
    """
    Extensions for caiman related functions
    """
    def __init__(self, series: pd.Series):
        if not configuration.HAS_CAIMAN:
            raise ImportError("Caiman not found in the environment")

        self._series = series

    def get_params(self) -> dict:
        params_path = \
            CURRENT_BATCH.joinpath(
                f"{self._series.uuid}.params"
            )

        return pickle.load(open(params_path, 'rb'))

    def get_mc_movie_path(self) -> pathlib.Path:
        if not self._series['module'] == 'caiman_motion_correction':
            raise TypeError("Not a motion correction batch item.")

        path = CURRENT_BATCH.joinpath(f"{self._series.uuid}_mc.tiff")

        if not path.is_file():
            raise FileNotFoundError("Motion corrected movie not found for this batch item.")

        return path

    def get_mc_movie(self):
        path = self.get_mc_movie_path()

        return cm.load_movie_chain([str(path)])

    def get_mc_eval_metrics(
            self,
            winsize: int = 100,
            swap_dim: bool = False,
            play_flow: bool = False,
            resize_fact_flow: float = 0.2
    ) -> dict:
        gSig_filt = self.get_params()['mc_kwargs']['gSig_filt']

        dims = self.get_mc_movie().shape[1:]

        template_metric, \
        correlations_metric, \
        flows_metric, \
        norms_metric, \
        crispness_metric = \
            cm.motion_correction.compute_metrics_motion_correction(
                self.get_mc_movie_path(),
                dims[0],
                dims[1],
                swap_dim,
                winsize=winsize,
                play_flow=play_flow,
                resize_fact_flow=resize_fact_flow,
                gSig_filt=gSig_filt
            )

        return \
            {
                "template_metric": template_metric,
                "correlations_metric": correlations_metric,
                "flows_metric": flows_metric,
                "norms_metric": norms_metric,
                "crispness_metric": crispness_metric
            }


#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Mon March 5 2017

@author: kushal

Chatzigeorgiou Group
Sars International Centre for Marine Molecular Biology

GNU GENERAL PUBLIC LICENSE Version 3, 29 June 2007
"""

# import sys
# sys.path.append('..')
from .pytemplates.plot_window_pytemplate import *
from .data_types import Transmission
from .history_widget import HistoryTreeWidget
from ..pyqtgraphCore import LinearRegionItem


class PlotWindow(QtWidgets.QMainWindow, Ui_MainWindow):
    def __init__(self, parent=None, *args):
        super().__init__()
        self.setupUi(self)
        self.setWindowTitle('Mesmerize - Plot')
        self.show()
        self.history_widget = HistoryTreeWidget(self)

    def set_history_widget(self, sources):
        self.dockWidgetTransmissions.setWidget(self.history_widget)
        self.history_widget.fill_widget(sources)
        self.dockWidgetTransmissions.show()

    def _set_curve_color(self):
        pass


class RegionSelectionPlot(PlotWindow):
    def __init__(self, parent=None, *args):
        PlotWindow.__init__(self)
        self.linear_region = None
        self.btnSetFo = QtWidgets.QPushButton(self)
        self.gridLayout.addItem(self.btnSetFo, 0, 0, 0, 0)

    def set_linear_region(self, bounds: list):
        self.linear_region = LinearRegionItem(bounds)
        self.linear_region.setZValue(-10)
        self.graphicsView.addItem(self.linear_region)

    def get_linear_region_bounds(self) -> tuple:
        return self.linear_region.get_region()



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file './ui_files/mainwindow.ui'
#
# Created by: PyQt5 UI code generator 5.9.2
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets

class Ui_MainWindow(object):
    def setupUi(self, MainWindow):
        MainWindow.setObjectName("MainWindow")
        MainWindow.resize(800, 600)
        self.centralwidget = QtWidgets.QWidget(MainWindow)
        self.centralwidget.setObjectName("centralwidget")
        MainWindow.setCentralWidget(self.centralwidget)
        self.menubar = QtWidgets.QMenuBar(MainWindow)
        self.menubar.setGeometry(QtCore.QRect(0, 0, 800, 30))
        self.menubar.setObjectName("menubar")
        self.menuDataframe = QtWidgets.QMenu(self.menubar)
        self.menuDataframe.setObjectName("menuDataframe")
        self.menuSave_current_tab = QtWidgets.QMenu(self.menuDataframe)
        self.menuSave_current_tab.setObjectName("menuSave_current_tab")
        self.menuSave_all_tabs = QtWidgets.QMenu(self.menuDataframe)
        self.menuSave_all_tabs.setObjectName("menuSave_all_tabs")
        self.menuView = QtWidgets.QMenu(self.menubar)
        self.menuView.setObjectName("menuView")
        self.menuTools = QtWidgets.QMenu(self.menubar)
        self.menuTools.setObjectName("menuTools")
        MainWindow.setMenuBar(self.menubar)
        self.statusbar = QtWidgets.QStatusBar(MainWindow)
        self.statusbar.setObjectName("statusbar")
        MainWindow.setStatusBar(self.statusbar)
        self.dockConsole = QtWidgets.QDockWidget(MainWindow)
        self.dockConsole.setMinimumSize(QtCore.QSize(68, 200))
        self.dockConsole.setFeatures(QtWidgets.QDockWidget.DockWidgetFloatable|QtWidgets.QDockWidget.DockWidgetMovable)
        self.dockConsole.setObjectName("dockConsole")
        self.dockWidgetContents = QtWidgets.QWidget()
        self.dockWidgetContents.setObjectName("dockWidgetContents")
        self.dockConsole.setWidget(self.dockWidgetContents)
        MainWindow.addDockWidget(QtCore.Qt.DockWidgetArea(8), self.dockConsole)
        self.actionDataframe_editor = QtWidgets.QAction(MainWindow)
        self.actionDataframe_editor.setObjectName("actionDataframe_editor")
        self.actionConsole = QtWidgets.QAction(MainWindow)
        self.actionConsole.setCheckable(True)
        self.actionConsole.setObjectName("actionConsole")
        self.actionCurrent_tab_filter_history = QtWidgets.QAction(MainWindow)
        self.actionCurrent_tab_filter_history.setObjectName("actionCurrent_tab_filter_history")
        self.actionUpdate_current_tab = QtWidgets.QAction(MainWindow)
        self.actionUpdate_current_tab.setObjectName("actionUpdate_current_tab")
        self.actionUpdate_all_tabs = QtWidgets.QAction(MainWindow)
        self.actionUpdate_all_tabs.setObjectName("actionUpdate_all_tabs")
        self.actionto_pickle_tab = QtWidgets.QAction(MainWindow)
        self.actionto_pickle_tab.setObjectName("actionto_pickle_tab")
        self.actionto_csv_tab = QtWidgets.QAction(MainWindow)
        self.actionto_csv_tab.setObjectName("actionto_csv_tab")
        self.actionto_excel_tab = QtWidgets.QAction(MainWindow)
        self.actionto_excel_tab.setObjectName("actionto_excel_tab")
        self.actionto_pickle = QtWidgets.QAction(MainWindow)
        self.actionto_pickle.setObjectName("actionto_pickle")
        self.actionto_excel = QtWidgets.QAction(MainWindow)
        self.actionto_excel.setObjectName("actionto_excel")
        self.actionto_csv = QtWidgets.QAction(MainWindow)
        self.actionto_csv.setObjectName("actionto_csv")
        self.actionLink_viewers = QtWidgets.QAction(MainWindow)
        self.actionLink_viewers.setObjectName("actionLink_viewers")
        self.actionSave_to_project = QtWidgets.QAction(MainWindow)
        self.actionSave_to_project.setObjectName("actionSave_to_project")
        self.actionViewCurrent_dataframe = QtWidgets.QAction(MainWindow)
        self.actionViewCurrent_dataframe.setObjectName("actionViewCurrent_dataframe")
        self.menuSave_current_tab.addSeparator()
        self.menuSave_current_tab.addAction(self.actionto_pickle_tab)
        self.menuSave_current_tab.addAction(self.actionto_csv_tab)
        self.menuSave_current_tab.addAction(self.actionto_excel_tab)
        self.menuSave_all_tabs.addAction(self.actionto_pickle)
        self.menuSave_all_tabs.addAction(self.actionto_csv)
        self.menuSave_all_tabs.addAction(self.actionto_excel)
        self.menuDataframe.addAction(self.actionDataframe_editor)
        self.menuDataframe.addSeparator()
        self.menuDataframe.addAction(self.actionUpdate_all_tabs)
        self.menuDataframe.addAction(self.actionUpdate_current_tab)
        self.menuDataframe.addSeparator()
        self.menuDataframe.addAction(self.menuSave_current_tab.menuAction())
        self.menuDataframe.addAction(self.menuSave_all_tabs.menuAction())
        self.menuDataframe.addSeparator()
        self.menuDataframe.addAction(self.actionSave_to_project)
        self.menuView.addAction(self.actionCurrent_tab_filter_history)
        self.menuView.addAction(self.actionViewCurrent_dataframe)
        self.menuView.addSeparator()
        self.menuView.addAction(self.actionConsole)
        self.menuTools.addAction(self.actionLink_viewers)
        self.menubar.addAction(self.menuDataframe.menuAction())
        self.menubar.addAction(self.menuView.menuAction())
        self.menubar.addAction(self.menuTools.menuAction())

        self.retranslateUi(MainWindow)
        self.actionConsole.toggled['bool'].connect(self.dockConsole.setVisible)
        QtCore.QMetaObject.connectSlotsByName(MainWindow)

    def retranslateUi(self, MainWindow):
        _translate = QtCore.QCoreApplication.translate
        MainWindow.setWindowTitle(_translate("MainWindow", "MainWindow"))
        self.menuDataframe.setTitle(_translate("MainWindow", "&Dataframe"))
        self.menuSave_current_tab.setTitle(_translate("MainWindow", "&Export current tab"))
        self.menuSave_all_tabs.setTitle(_translate("MainWindow", "&Export root"))
        self.menuView.setTitle(_translate("MainWindow", "&View"))
        self.menuTools.setTitle(_translate("MainWindow", "&Tools"))
        self.dockConsole.setWindowTitle(_translate("MainWindow", "&Console: Project Browser"))
        self.actionDataframe_editor.setText(_translate("MainWindow", "&Dataframe editor"))
        self.actionConsole.setText(_translate("MainWindow", "&Console"))
        self.actionCurrent_tab_filter_history.setText(_translate("MainWindow", "Current tab &filter history"))
        self.actionUpdate_current_tab.setText(_translate("MainWindow", "Update &current tab"))
        self.actionUpdate_all_tabs.setText(_translate("MainWindow", "&Update all tabs"))
        self.actionto_pickle_tab.setText(_translate("MainWindow", "&to pickle"))
        self.actionto_pickle_tab.setToolTip(_translate("MainWindow", "Save current tab as a python pickle"))
        self.actionto_csv_tab.setText(_translate("MainWindow", "to &csv"))
        self.actionto_excel_tab.setText(_translate("MainWindow", "to &excel"))
        self.actionto_pickle.setText(_translate("MainWindow", "&to pickle"))
        self.actionto_excel.setText(_translate("MainWindow", "to &excel"))
        self.actionto_csv.setText(_translate("MainWindow", "to &csv"))
        self.actionLink_viewers.setText(_translate("MainWindow", "&Link viewers"))
        self.actionSave_to_project.setText(_translate("MainWindow", "Save to &project"))
        self.actionViewCurrent_dataframe.setText(_translate("MainWindow", "Current &dataframe"))




#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on May 15 2018

@author: kushal

Chatzigeorgiou Group
Sars International Centre for Marine Molecular Biology

GNU GENERAL PUBLIC LICENSE Version 3, 29 June 2007

Adapted from @agiovann and @epnev
"""

from __future__ import division
import sys
import cv2
try:
    cv2.setNumThreads(1)
except:
    print('Open CV is naturally single threaded')
import caiman as cm
import numpy as np
import os
import pickle
import json
from caiman.motion_correction import MotionCorrect
import tifffile
import traceback
# import numba
from glob import glob
from time import time
import logging

if not sys.argv[0] == __file__:
    from ...core.common import ViewerUtils
    from ...core.viewer_work_environment import ViewerWorkEnv


def run(batch_dir: str, UUID: str):
    logging.basicConfig(stream=sys.stdout, level=logging.DEBUG,
                        format="%(relativeCreated)12d [%(filename)s:%(funcName)20s():%(lineno)s] [%(process)d] %(message)s")
    start_time = time()

    output = {'status': 0, 'output_info': ''}
    file_path = os.path.join(batch_dir, UUID)

    input_params = pickle.load(open(file_path + '.params', 'rb'))

    try:
        if 'is_3d' in input_params.keys():
            if input_params['is_3d']:
                run_multi(batch_dir, UUID, output)
            else:
                run_single(batch_dir, UUID, output)
        else:
            run_single(batch_dir, UUID, output)

    except Exception as e:
        output.update({'status': 0, 'output_info': traceback.format_exc()})

    end_time = time()
    processing_time = (end_time - start_time) / 60

    output.update({'processing_time': processing_time})

    json.dump(output, open(file_path + '.out', 'w'))


def run_single(batch_dir, UUID, output):
    file_path = os.path.join(batch_dir, UUID)

    n_processes = os.environ['_MESMERIZE_N_THREADS']
    n_processes = int(n_processes)

    c, dview, n_processes = cm.cluster.setup_cluster(
        backend='local', n_processes=n_processes, single_thread=False, ignore_preexisting=True
    )

    fname = [file_path + '_input.tiff']
    # TODO: Should just unpack the input params as kwargs
    input_params: dict = pickle.load(open(file_path + '.params', 'rb'))
    mc_kwargs = input_params['mc_kwargs']

    splits_rig = n_processes

    splits_els = n_processes

    if os.environ['_MESMERIZE_USE_CUDA'] == 'True':
        USE_CUDA = True
    else:
        USE_CUDA = False

    min_mov = cm.load(fname[0], subindices=range(200)).min()

    mc = MotionCorrect(
        fname[0], min_mov,
        dview=dview,
        splits_rig=splits_rig,
        splits_els=splits_els,
        shifts_opencv=True,
        nonneg_movie=True,
        use_cuda=USE_CUDA,
        **mc_kwargs
    )

    if 'template' in input_params.keys():
        template = input_params['template']
    else:
        template = None

    mc.motion_correct_pwrigid(save_movie=True, template=template)
    m_els = cm.load(mc.fname_tot_els)
    bord_px_els = np.ceil(np.maximum(np.max(np.abs(mc.x_shifts_els)),
                                     np.max(np.abs(mc.y_shifts_els)))).astype(np.int)

    m_els -= np.nanmin(m_els)

    if input_params['output_bit_depth'] == 'Do not convert':
        pass
    elif input_params['output_bit_depth'] == '8':
        m_els = m_els.astype(np.uint8, copy=False)
    elif input_params['output_bit_depth'] == '16':
        m_els = m_els.astype(np.uint16)

    img_out_path = os.path.join(batch_dir, f'{UUID}_mc.tiff')
    tifffile.imsave(img_out_path, m_els, bigtiff=True, imagej=True, compress=1)
    output['output_files'] = [UUID + '_mc.tiff']

    output.update({'status': 1, 'bord_px': int(bord_px_els)})

    if not input_params.get('keep_memmap', False):
        for mf in glob(os.path.join(batch_dir, UUID + '*.mmap')):
            try:
                os.remove(mf)
            except:
                pass

    dview.terminate()

    return output


def run_multi(batch_dir, UUID, output):
    file_path = os.path.join(batch_dir, UUID)

    n_processes = os.environ['_MESMERIZE_N_THREADS']
    n_processes = int(n_processes)

    filename = [file_path + '_input.tiff']

    seq = tifffile.TiffFile(filename[0]).asarray()
    seq_shape = seq.shape

    # assume default tzxy
    for z in range(seq.shape[1]):
        tifffile.imsave(f'{file_path}_z{z}.tiff', seq[:, z, :, :])

    del seq

    print("******** Creating process pool ********")
    c, dview, n_processes = cm.cluster.setup_cluster(
        backend='local', n_processes=n_processes, single_thread=False, ignore_preexisting=True
    )

    # TODO: Should just unpack the input params as kwargs
    input_params = pickle.load(open(file_path + '.params', 'rb'))
    mc_kwargs = input_params['mc_kwargs']

    splits_rig = n_processes
    splits_els = n_processes

    if os.environ['_MESMERIZE_USE_CUDA'] == 'True':
        USE_CUDA = True
    else:
        USE_CUDA = False

    output_files = []
    for z in range(seq_shape[1]):
        print(f"Plane {z} / {seq_shape[1]}")
        filename = [f'{file_path}_z{z}.tiff']
        print('Creating memmap')

        min_mov = cm.load(filename[0], subindices=range(200)).min()

        mc = MotionCorrect(
            filename[0], min_mov,
            dview=dview,
            splits_rig=splits_rig,
            splits_els=splits_els,
            shifts_opencv=True,
            nonneg_movie=True,
            use_cuda=USE_CUDA,
            **mc_kwargs
        )

        mc.motion_correct_pwrigid(save_movie=True)
        m_els = cm.load(mc.fname_tot_els)
        bord_px_els = np.ceil(np.maximum(np.max(np.abs(mc.x_shifts_els)),
                                         np.max(np.abs(mc.y_shifts_els)))).astype(np.int)

        m_els -= np.nanmin(m_els)

        if input_params['output_bit_depth'] == 'Do not convert':
            pass
        elif input_params['output_bit_depth'] == '8':
            m_els = m_els.astype(np.uint8, copy=False)
        elif input_params['output_bit_depth'] == '16':
            m_els = m_els.astype(np.uint16)

        if z == 0:
            mc_out = np.zeros(seq_shape, dtype=m_els.dtype)

        mc_out[:, z, :, :] = m_els

    img_out_path = os.path.join(batch_dir, f'{UUID}_mc.tiff')
    tifffile.imsave(img_out_path, mc_out, bigtiff=True, imagej=True, compress=1)
    output['output_files'] = [f'{UUID}_mc.tiff']

    output.update(
        {
            'status': 1,
            'bord_px': int(bord_px_els),
        }
    )

    for mf in glob(os.path.join(batch_dir, UUID + '*.mmap')):
        try:
            os.remove(mf)
        except:
            pass

    dview.terminate()

    return output


class Output:
    def __init__(self, batch_path, UUID, viewer_ref):
        vi = ViewerUtils(viewer_ref)

        if not vi.discard_workEnv():
            return

        vi.viewer.status_bar_label.showMessage('Please wait, loading motion corrected image sequence...')

        pik_path = os.path.join(batch_path, f'{UUID}_input.pik')
        tiff_path = os.path.join(batch_path, f'{UUID}_mc.tiff')

        vi.viewer.workEnv = ViewerWorkEnv.from_pickle(pik_path, tiff_path)
        #tiff_path = batch_path + '/' + str(UUID) + '_mc.tiff'
        #workEnv.imgdata.seq = tifffile.imread(tiff_path).T
        vi.update_workEnv()
        vi.viewer.status_bar_label.showMessage('Finished loading motion corrected image sequence!')

        input_params = pickle.load(
            open(
                os.path.join(batch_path, f'{UUID}.params'),
                'rb'
            )
        )

        name = input_params['item_name']
        vi.viewer.ui.label_curr_img_seq_name.setText('MotCor :' + name)

        out_file = json.load(
            open(os.path.join(batch_path, f'{UUID}.out'), 'r')
        )

        bpx = out_file['bord_px']
        input_params.update({'bord_px': bpx})

        vi.viewer.workEnv.history_trace.append({'caiman_motion_correction': input_params})

        vi.enable_ui(True)


class BitDepthConverter:
    """
    Downscale the bit depth of image to uint8, uint16, or uint32 using a Look up table.
    Usage example:

    First create a LUT (look up table) of the bit depth you desire. Pass the min and max levels as a tuple or list:

    lut_8_bit = create_lut(levels=[32, 2049], source=16, output=8)

    Now use this LUT to downscale an image:

    downscaled_img = apply_lut(img, lut_8_bit)
    """
    @staticmethod
    def create_lut(levels, source, out):
        """
        :param levels:      min and max levels with which to create the LUT
        :type levels:       tuple or list
        :param source:      bit depth of the source. 16, 32, or 64
        :type source:       int
        :param output:      desired output bit depth
        :type output:       int
        :return:            LUT (Look up table) to use for downscaling the bit depth
        :rtype:             np.ndarray
        """

        accepted_srcs = [16, 32, 64]
        if source not in accepted_srcs:
            raise TypeError('Can only convert from uint16, uint32, or uint64')

        accepted_outs = [8, 16, 32]
        if out not in accepted_outs:
            raise TypeError('Can only output uint8, uint16, or uint32')

        type_str = 'uint' + str(source)
        lut = np.arange(2**source, dtype=type_str)
        lut.clip(levels[0], levels[1], out=lut)
        lut -= levels[0]
        np.floor_divide(lut, (levels[1] - levels[0] + 1) / (2**out), out=lut, casting='unsafe')

        type_str = 'uint' + str(out)
        return lut.astype(type_str)

    @staticmethod
    def apply_lut(image, lut):
        """
        :param image:   The image upon which to apply the LUT (Look up table) and change its bit depth
        :type image:    np.ndarray
        :param lut:     The LUT to use for downscaling the bit depth. Generated by BitDepthConvert.create_LUT
        :type lut:      np.ndarray
        :return:        Downscaled image with the LUT (Look up table) applied to it
        :rtype:         np.ndarray
        """
        return np.take(lut, image).astype(lut.dtype)

#if sys.argv[0] == __file__:
if __name__ == '__main__':
    run(sys.argv[1], sys.argv[2])



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on June 1 2018

@author: kushal

Chatzigeorgiou Group
Sars International Centre for Marine Molecular Biology

GNU GENERAL PUBLIC LICENSE Version 3, 29 June 2007
"""

from ...pyqtgraphCore.Qt import QtCore, QtGui, QtWidgets
from skimage.transform import rescale
import numpy as np
from ..core.common import ViewerUtils
from functools import partial
from multiprocessing.pool import ThreadPool as Pool
from ...common import get_sys_config
import traceback


class ResizeDialogBox(QtWidgets.QWidget):
    sig_increase_progressBar = QtCore.pyqtSignal()

    def __init__(self, viewer_interface):
        QtWidgets.QWidget.__init__(self)
        assert isinstance(viewer_interface, ViewerUtils)
        self.vi = viewer_interface
        layout = QtWidgets.QVBoxLayout()

        label = QtWidgets.QLabel()
        label.setText('Set scaling factor in %')
        layout.addWidget(label)

        self.spinBox = QtWidgets.QSpinBox()
        self.spinBox.setMaximum(100)
        self.spinBox.setMinimum(1)
        self.spinBox.setValue(60)
        layout.addWidget(self.spinBox)

        self.btn = QtWidgets.QPushButton()
        self.btn.setText('Apply')
        layout.addWidget(self.btn)
        self.progressBar = QtWidgets.QProgressBar()
        self.progressBar.setValue(0)
        self.progressBar.setMaximum(0)
        self.progressBar.setMaximum(100)
        layout.addWidget(self.progressBar)
        self.status_label = QtWidgets.QLabel()
        self.status_label.setText('')
        layout.addWidget(self.status_label)
        self.btn.clicked.connect(partial(self.resize_img_seq, self.spinBox.value() / 100))
        self.setLayout(layout)
        self.setWindowTitle('Resize')
        self.frames_processed = 0

    def resize_img_seq(self, factor):
        if self.vi.viewer.workEnv.isEmpty:
            QtWidgets.QMessageBox.warning(self, 'Cannot resize', 'Your work environment is empty, nothing to resize!')
            self.hide()
            return
        self.vi.viewer.status_bar_label.showMessage('Resizing, please wait...')
        n_processes = int(get_sys_config()['_MESMERIZE_N_THREADS'])

        seq = self.vi.viewer.workEnv.imgdata.seq
        self.num_frames_to_process = seq.shape[2]
        self.frames_processed = 0

        resizer = ResizeObject(seq, factor, n_processes)

        resizer.signals.frame_processed.connect(self.increase_progress_bar)
        resizer.signals.result.connect(self.set_resized_array)
        resizer.signals.error.connect(self.show_error_message)
        resizer.signals.finished.connect(lambda: self.vi.viewer.status_bar_label.showMessage('Resize completed!'))
        resizer.signals.finished.connect(lambda: self.status_label.setText('Done!'))
        resizer.signals.finished.connect(lambda: self.progressBar.setValue(0))

        self.thread_pool = QtCore.QThreadPool()
        self.thread_pool.setMaxThreadCount(n_processes)
        self.thread_pool.start(resizer)

    @QtCore.pyqtSlot()
    def increase_progress_bar(self):
        self.frames_processed += 1
        self.progressBar.setValue(int(self.frames_processed * 100 / self.num_frames_to_process))

    @QtCore.pyqtSlot(np.ndarray)
    def set_resized_array(self, array):
        self.vi.viewer.workEnv.imgdata.seq = array
        self.vi.update_workEnv()

    @QtCore.pyqtSlot(str)
    def show_error_message(self, error_msg):
        QtWidgets.QMessageBox.warning(self, 'Error', 'The following error occured while resizing:\n' + str(error_msg))
        self.vi.viewer.status_bar_label.clearMessage()


class Signals(QtCore.QObject):
    frame_processed = QtCore.pyqtSignal()
    result = QtCore.pyqtSignal(np.ndarray)
    finished = QtCore.pyqtSignal()
    error = QtCore.pyqtSignal(str)


class ResizeObject(QtCore.QRunnable):
    def __init__(self, seq, factor, n_processes):
        super(ResizeObject, self).__init__()
        self.signals = Signals()
        self.seq = seq
        self.factor = factor
        self.n_processes = n_processes

    def resize_split(self, template_factor, img):
        tmpl = template_factor[0]
        factor = template_factor[1]
        r = np.zeros((tmpl.shape[0], tmpl.shape[1], img.shape[2]), dtype=img.dtype)

        for i in range(0, img.shape[2]):
            r[:, :, i] = rescale(img[:, :, i], factor, preserve_range=True)
            self.signals.frame_processed.emit()

        return r

    def run(self):
        try:
            tmpl = rescale(self.seq[:, :, :2], self.factor)

            template_factor = [tmpl, self.factor]

            pool = Pool(self.n_processes)
            splits_resized = pool.map(partial(self.resize_split, template_factor),
                                      np.array_split(self.seq, self.n_processes, axis=2))

            resized_array = np.concatenate(splits_resized, axis=2)
        except:
            self.signals.error.emit(str(traceback.format_exc()))
        else:
            self.signals.result.emit(resized_array)
        finally:
            self.signals.finished.emit()



# from ...pyqtgraphCore.widgets.MatplotlibWidget import MatplotlibWidget
from ...pyqtgraphCore.imageview import ImageView
import numpy as np


def display_projection(projection: str, imgseq: np.ndarray, img_name: str):
    # Due to weird importing you must do this, not a big deal
    iv = ImageView.ImageView()
    # ax = mw.fig.add_subplot(111)

    if projection == 'mean':
        t = 'Mean Projection of : ' + img_name
        p = imgseq.mean(axis=2)

    elif projection == 'max':
        t = 'Max Projection of : ' + img_name
        p = imgseq.max(axis=2)

    elif projection == 'std':
        t = 'Std. Deviation projection of : ' + img_name
        p = imgseq.std(axis=2)

    else:
        raise ValueError('Invalid projection type, only accepts "mean", "max", or "std"')

    # ax.set_title(t)
    iv.setImage(p.T)
    iv.ui.label_curr_img_seq_name.setText(t)
    iv.setWindowTitle(t)
    # ax.imshow(p.T)

    iv.show()
    return iv



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file './project_config_pytemplate.ui'
#
# Created by: PyQt5 UI code generator 5.9.2
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets

class Ui_Form(object):
    def setupUi(self, Form):
        Form.setObjectName("Form")
        Form.resize(795, 610)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(Form.sizePolicy().hasHeightForWidth())
        Form.setSizePolicy(sizePolicy)
        Form.setMinimumSize(QtCore.QSize(593, 610))
        Form.setMaximumSize(QtCore.QSize(2000, 2000))
        self.gridLayout = QtWidgets.QGridLayout(Form)
        self.gridLayout.setObjectName("gridLayout")
        self.verticalLayout_2 = QtWidgets.QVBoxLayout()
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.horizontalLayout_3 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_3.setObjectName("horizontalLayout_3")
        spacerItem = QtWidgets.QSpacerItem(1, 10, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_3.addItem(spacerItem)
        self.label_4 = QtWidgets.QLabel(Form)
        self.label_4.setObjectName("label_4")
        self.horizontalLayout_3.addWidget(self.label_4)
        spacerItem1 = QtWidgets.QSpacerItem(1, 10, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_3.addItem(spacerItem1)
        self.verticalLayout_2.addLayout(self.horizontalLayout_3)
        self.listwROIDefs = QtWidgets.QListWidget(Form)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.listwROIDefs.sizePolicy().hasHeightForWidth())
        self.listwROIDefs.setSizePolicy(sizePolicy)
        self.listwROIDefs.setAlternatingRowColors(True)
        self.listwROIDefs.setObjectName("listwROIDefs")
        self.verticalLayout_2.addWidget(self.listwROIDefs)
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.lineEdNewROIDef = QtWidgets.QLineEdit(Form)
        self.lineEdNewROIDef.setObjectName("lineEdNewROIDef")
        self.horizontalLayout_2.addWidget(self.lineEdNewROIDef)
        self.btnAddNewROICol = QtWidgets.QPushButton(Form)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.btnAddNewROICol.sizePolicy().hasHeightForWidth())
        self.btnAddNewROICol.setSizePolicy(sizePolicy)
        self.btnAddNewROICol.setMaximumSize(QtCore.QSize(51, 16777215))
        self.btnAddNewROICol.setObjectName("btnAddNewROICol")
        self.horizontalLayout_2.addWidget(self.btnAddNewROICol)
        self.verticalLayout_2.addLayout(self.horizontalLayout_2)
        self.gridLayout.addLayout(self.verticalLayout_2, 0, 2, 1, 1)
        self.horizontalLayout_5 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_5.setObjectName("horizontalLayout_5")
        self.gridLayout_3 = QtWidgets.QGridLayout()
        self.gridLayout_3.setObjectName("gridLayout_3")
        spacerItem2 = QtWidgets.QSpacerItem(1, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.gridLayout_3.addItem(spacerItem2, 0, 0, 1, 1)
        self.label_2 = QtWidgets.QLabel(Form)
        self.label_2.setObjectName("label_2")
        self.gridLayout_3.addWidget(self.label_2, 0, 1, 1, 1)
        spacerItem3 = QtWidgets.QSpacerItem(1, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.gridLayout_3.addItem(spacerItem3, 0, 2, 1, 1)
        self.listwInclude = QtWidgets.QListWidget(Form)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.listwInclude.sizePolicy().hasHeightForWidth())
        self.listwInclude.setSizePolicy(sizePolicy)
        self.listwInclude.setDragEnabled(True)
        self.listwInclude.setDragDropOverwriteMode(False)
        self.listwInclude.setDragDropMode(QtWidgets.QAbstractItemView.DragDrop)
        self.listwInclude.setDefaultDropAction(QtCore.Qt.MoveAction)
        self.listwInclude.setAlternatingRowColors(True)
        self.listwInclude.setSelectionMode(QtWidgets.QAbstractItemView.ExtendedSelection)
        self.listwInclude.setObjectName("listwInclude")
        self.gridLayout_3.addWidget(self.listwInclude, 1, 0, 1, 3)
        self.horizontalLayout_5.addLayout(self.gridLayout_3)
        self.gridLayout_4 = QtWidgets.QGridLayout()
        self.gridLayout_4.setObjectName("gridLayout_4")
        spacerItem4 = QtWidgets.QSpacerItem(1, 10, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.gridLayout_4.addItem(spacerItem4, 0, 2, 1, 1)
        self.label_3 = QtWidgets.QLabel(Form)
        self.label_3.setObjectName("label_3")
        self.gridLayout_4.addWidget(self.label_3, 0, 1, 1, 1)
        spacerItem5 = QtWidgets.QSpacerItem(1, 10, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.gridLayout_4.addItem(spacerItem5, 0, 0, 1, 1)
        self.listwExclude = QtWidgets.QListWidget(Form)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.listwExclude.sizePolicy().hasHeightForWidth())
        self.listwExclude.setSizePolicy(sizePolicy)
        self.listwExclude.setDragEnabled(True)
        self.listwExclude.setDragDropOverwriteMode(False)
        self.listwExclude.setDragDropMode(QtWidgets.QAbstractItemView.DragDrop)
        self.listwExclude.setDefaultDropAction(QtCore.Qt.MoveAction)
        self.listwExclude.setAlternatingRowColors(True)
        self.listwExclude.setSelectionMode(QtWidgets.QAbstractItemView.ExtendedSelection)
        self.listwExclude.setObjectName("listwExclude")
        self.gridLayout_4.addWidget(self.listwExclude, 1, 0, 1, 3)
        self.horizontalLayout_5.addLayout(self.gridLayout_4)
        self.gridLayout.addLayout(self.horizontalLayout_5, 0, 0, 1, 2)
        self.groupBox = QtWidgets.QGroupBox(Form)
        self.groupBox.setAlignment(QtCore.Qt.AlignCenter)
        self.groupBox.setObjectName("groupBox")
        self.gridLayout_2 = QtWidgets.QGridLayout(self.groupBox)
        self.gridLayout_2.setObjectName("gridLayout_2")
        self.btnAddCustomColumn = QtWidgets.QPushButton(self.groupBox)
        self.btnAddCustomColumn.setObjectName("btnAddCustomColumn")
        self.gridLayout_2.addWidget(self.btnAddCustomColumn, 7, 0, 1, 1)
        self.radioButtonStr = QtWidgets.QRadioButton(self.groupBox)
        self.radioButtonStr.setObjectName("radioButtonStr")
        self.gridLayout_2.addWidget(self.radioButtonStr, 2, 0, 1, 1)
        self.label_7 = QtWidgets.QLabel(self.groupBox)
        self.label_7.setObjectName("label_7")
        self.gridLayout_2.addWidget(self.label_7, 1, 0, 1, 1)
        self.radioButtonInt64 = QtWidgets.QRadioButton(self.groupBox)
        self.radioButtonInt64.setObjectName("radioButtonInt64")
        self.gridLayout_2.addWidget(self.radioButtonInt64, 3, 0, 1, 1)
        self.radioButtonFloat64 = QtWidgets.QRadioButton(self.groupBox)
        self.radioButtonFloat64.setObjectName("radioButtonFloat64")
        self.gridLayout_2.addWidget(self.radioButtonFloat64, 4, 0, 1, 1)
        self.lineEditCustomColumnReplacementValue = QtWidgets.QLineEdit(self.groupBox)
        self.lineEditCustomColumnReplacementValue.setText("")
        self.lineEditCustomColumnReplacementValue.setObjectName("lineEditCustomColumnReplacementValue")
        self.gridLayout_2.addWidget(self.lineEditCustomColumnReplacementValue, 6, 0, 1, 1)
        self.lineEditCustomColumnName = QtWidgets.QLineEdit(self.groupBox)
        self.lineEditCustomColumnName.setObjectName("lineEditCustomColumnName")
        self.gridLayout_2.addWidget(self.lineEditCustomColumnName, 0, 0, 1, 1)
        self.radioButtonBoolean = QtWidgets.QRadioButton(self.groupBox)
        self.radioButtonBoolean.setEnabled(False)
        self.radioButtonBoolean.setObjectName("radioButtonBoolean")
        self.gridLayout_2.addWidget(self.radioButtonBoolean, 5, 0, 1, 1)
        self.gridLayout.addWidget(self.groupBox, 2, 2, 1, 1)
        self.horizontalLayout_8 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_8.setObjectName("horizontalLayout_8")
        spacerItem6 = QtWidgets.QSpacerItem(338, 13, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_8.addItem(spacerItem6)
        self.horizontalLayout_6 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_6.setObjectName("horizontalLayout_6")
        self.btnClose = QtWidgets.QPushButton(Form)
        self.btnClose.setObjectName("btnClose")
        self.horizontalLayout_6.addWidget(self.btnClose)
        self.btnSave = QtWidgets.QPushButton(Form)
        self.btnSave.setMinimumSize(QtCore.QSize(92, 0))
        self.btnSave.setObjectName("btnSave")
        self.horizontalLayout_6.addWidget(self.btnSave)
        self.horizontalLayout_8.addLayout(self.horizontalLayout_6)
        self.gridLayout.addLayout(self.horizontalLayout_8, 3, 0, 1, 3)
        self.verticalLayout = QtWidgets.QVBoxLayout()
        self.verticalLayout.setObjectName("verticalLayout")
        self.horizontalLayout_4 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_4.setObjectName("horizontalLayout_4")
        spacerItem7 = QtWidgets.QSpacerItem(1, 10, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_4.addItem(spacerItem7)
        self.label_5 = QtWidgets.QLabel(Form)
        self.label_5.setObjectName("label_5")
        self.horizontalLayout_4.addWidget(self.label_5)
        spacerItem8 = QtWidgets.QSpacerItem(1, 10, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_4.addItem(spacerItem8)
        self.verticalLayout.addLayout(self.horizontalLayout_4)
        self.listwStimDefs = QtWidgets.QListWidget(Form)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.listwStimDefs.sizePolicy().hasHeightForWidth())
        self.listwStimDefs.setSizePolicy(sizePolicy)
        self.listwStimDefs.setAlternatingRowColors(True)
        self.listwStimDefs.setObjectName("listwStimDefs")
        self.verticalLayout.addWidget(self.listwStimDefs)
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.lineEdNewStimCol = QtWidgets.QLineEdit(Form)
        self.lineEdNewStimCol.setObjectName("lineEdNewStimCol")
        self.horizontalLayout.addWidget(self.lineEdNewStimCol)
        self.btnAddNewStimCol = QtWidgets.QPushButton(Form)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.btnAddNewStimCol.sizePolicy().hasHeightForWidth())
        self.btnAddNewStimCol.setSizePolicy(sizePolicy)
        self.btnAddNewStimCol.setMaximumSize(QtCore.QSize(51, 16777215))
        self.btnAddNewStimCol.setObjectName("btnAddNewStimCol")
        self.horizontalLayout.addWidget(self.btnAddNewStimCol)
        self.verticalLayout.addLayout(self.horizontalLayout)
        self.gridLayout.addLayout(self.verticalLayout, 2, 0, 1, 1)
        self.verticalLayout_3 = QtWidgets.QVBoxLayout()
        self.verticalLayout_3.setObjectName("verticalLayout_3")
        self.horizontalLayout_9 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_9.setObjectName("horizontalLayout_9")
        spacerItem9 = QtWidgets.QSpacerItem(1, 10, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_9.addItem(spacerItem9)
        self.label_6 = QtWidgets.QLabel(Form)
        self.label_6.setObjectName("label_6")
        self.horizontalLayout_9.addWidget(self.label_6)
        spacerItem10 = QtWidgets.QSpacerItem(1, 10, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_9.addItem(spacerItem10)
        self.verticalLayout_3.addLayout(self.horizontalLayout_9)
        self.listwCustomColumns = QtWidgets.QListWidget(Form)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.listwCustomColumns.sizePolicy().hasHeightForWidth())
        self.listwCustomColumns.setSizePolicy(sizePolicy)
        self.listwCustomColumns.setAlternatingRowColors(True)
        self.listwCustomColumns.setObjectName("listwCustomColumns")
        self.verticalLayout_3.addWidget(self.listwCustomColumns)
        self.gridLayout.addLayout(self.verticalLayout_3, 2, 1, 1, 1)
        self.line = QtWidgets.QFrame(Form)
        self.line.setFrameShape(QtWidgets.QFrame.HLine)
        self.line.setFrameShadow(QtWidgets.QFrame.Sunken)
        self.line.setObjectName("line")
        self.gridLayout.addWidget(self.line, 1, 0, 1, 3)

        self.retranslateUi(Form)
        self.lineEdNewROIDef.returnPressed.connect(self.btnAddNewROICol.click)
        self.lineEdNewStimCol.returnPressed.connect(self.btnAddNewStimCol.click)
        QtCore.QMetaObject.connectSlotsByName(Form)
        Form.setTabOrder(self.listwInclude, self.listwExclude)
        Form.setTabOrder(self.listwExclude, self.listwROIDefs)
        Form.setTabOrder(self.listwROIDefs, self.listwStimDefs)
        Form.setTabOrder(self.listwStimDefs, self.lineEdNewROIDef)
        Form.setTabOrder(self.lineEdNewROIDef, self.btnAddNewROICol)
        Form.setTabOrder(self.btnAddNewROICol, self.lineEdNewStimCol)
        Form.setTabOrder(self.lineEdNewStimCol, self.btnAddNewStimCol)
        Form.setTabOrder(self.btnAddNewStimCol, self.btnSave)
        Form.setTabOrder(self.btnSave, self.btnClose)

    def retranslateUi(self, Form):
        _translate = QtCore.QCoreApplication.translate
        Form.setWindowTitle(_translate("Form", "Dataframe Config"))
        self.label_4.setText(_translate("Form", "ROI Type Columns"))
        self.lineEdNewROIDef.setPlaceholderText(_translate("Form", "Enter New ROI Column Name"))
        self.btnAddNewROICol.setText(_translate("Form", "Add"))
        self.label_2.setText(_translate("Form", "Visible in Project Browser"))
        self.label_3.setText(_translate("Form", "Hidden in Project Browser"))
        self.groupBox.setTitle(_translate("Form", "Add new custom column"))
        self.btnAddCustomColumn.setText(_translate("Form", "Add"))
        self.radioButtonStr.setText(_translate("Form", "standard te&xt (str)"))
        self.label_7.setText(_translate("Form", "Choose data type:"))
        self.radioButtonInt64.setText(_translate("Form", "whole &numbers (np.int64)"))
        self.radioButtonFloat64.setText(_translate("Form", "deci&mal numbers (np.float64)"))
        self.lineEditCustomColumnReplacementValue.setToolTip(_translate("Form", "If the project dataframe is not empty this value will be set in all rows of the new column"))
        self.lineEditCustomColumnReplacementValue.setPlaceholderText(_translate("Form", "Dataframe replacement value"))
        self.lineEditCustomColumnName.setToolTip(_translate("Form", "Name for the new column"))
        self.lineEditCustomColumnName.setPlaceholderText(_translate("Form", "Column name"))
        self.radioButtonBoolean.setText(_translate("Form", "boolean (True/False &values)"))
        self.btnClose.setText(_translate("Form", "Close"))
        self.btnSave.setText(_translate("Form", "Save and apply"))
        self.label_5.setText(_translate("Form", "Stimulus/Behavior Type Columns"))
        self.lineEdNewStimCol.setPlaceholderText(_translate("Form", "Enter New Stim Column Name"))
        self.btnAddNewStimCol.setText(_translate("Form", "Add"))
        self.label_6.setText(_translate("Form", "Custom Columns"))




import numpy as np
from . import cross_correlation
from . import tvregdiff
from . import sosd


def modln(x): return np.sign(x) * np.log(np.abs(x))
def modlog10(x): np.sign(x) * np.log10(np.abs(x))
def normalize(a): return ((a - np.min(a)) / (np.max(a - np.min(a))))
